library(spam)
#library(GPBayes)
library(gsl)

logit = function(x, lb, ub){
	y = log(x-lb) - log(ub-x)
	return(y)
}

ilogit = function(y, lb, ub){
	x =  lb + (exp(y)/(1+exp(y))) * (ub-lb)
	return(x)
}

CH <- function(d, beta, alpha, nu){


  const = lgamma(nu+alpha) - lgamma(nu)
  out = exp(const) * gsl::hyperg_U(alpha, 1-nu, nu*d^2/beta) 
  #out = exp(const) * GPBayes::HypergU(alpha, 1-nu, nu*(d/beta)^2) 

  out[d==0] = 1 
  return(out)
}


matern <- function(d, phi, nu){
  
  if(nu==0.5){
    out = exp(-d/phi)
  }else{
    tau = (2*nu)^0.5*d/phi 
    out = (2^(1-nu) / gamma(nu)) * tau^nu * besselK(tau, nu)
    out[d==0] = 1
  }
  

  return(out)
}



rdist.chord = function(coor1, coor2){
xlat1=coor1[ ,2] * pi / 180 
ylon1=coor1[ ,1] * pi / 180

xlat2=coor2[, 2] * pi / 180
ylon2=coor2[ ,1] * pi / 180

Earth_radius=6371.0087714 #WGS84 mean radius

n = dim(coor1)[1]
m = dim(coor2)[1]
d = matrix(NA, n, m)
for(i in 1:n){
	for(j in 1:m){
		temp1 = (sin((xlat1[i]-xlat2[j])/2))^2
		temp2 = (sin((ylon1[i]-ylon2[j])/2))^2
		d[i,j] = 2*Earth_radius*asin(sqrt(temp1 + cos(xlat1[i])*cos(xlat2[j])*temp2))

	}
}


return(d)
}

###################################################################################
############## Maximization with REML 
###################################################################################

prologlik_matern = function(param, y, H, d, nu){
  

n = length(y)
q = dim(H)[2]


phi = exp(param[1])
sig2eps = ilogit(param[2], 0, 1)
cormat = matern(d, phi, nu=nu)  


Q = chol(cormat + diag(sig2eps,n))

QInv = chol2inv(Q)

HRH = t(H)%*%QInv%*%H
bhat = solve(HRH, t(H)%*%(QInv%*%y))
ytilde = y - H%*%bhat 

sig2hat = t(ytilde)%*%QInv%*%ytilde/(n-q) 


negloglik = (n-q)*log(2*pi*sig2hat) + 2*sum(log(diag(Q))) + 2*sum(log(diag(chol(HRH)))) + 1/(n-q)

  
return(-0.5*c(negloglik))

}

prologlik_CH = function(param, y, H, d, nu){
  
# bounds for beta 
e0 = 1e-20
e1 = 1e8
# bounds for alpha
a0 = 1e-5
a1 = 6



n = length(y)
q = dim(H)[2]


# beta = exp(param[1])
# alpha = 1.0 + exp(param[2])

beta = e0 + (exp(param[1])/(1+exp(param[1]))) * (e1-e0)
alpha = a0 + (exp(param[2])/(1+exp(param[2]))) * (a1-a0)
sig2eps = ilogit(param[3], 0, 1)
cormat = CH(d, beta, alpha, nu=nu)
  

Q = chol(cormat + diag(sig2eps,n))

QInv = chol2inv(Q)

HRH = t(H)%*%QInv%*%H
bhat = solve(HRH, t(H)%*%(QInv%*%y))
ytilde = y - H%*%bhat 

sig2hat = t(ytilde)%*%QInv%*%ytilde/(n-q) 


negloglik = (n-q)*log(2*pi*sig2hat) + 2*sum(log(diag(Q))) + 2*sum(log(diag(chol(HRH)))) + 1/(n-q)

  
return(-0.5*c(negloglik))

}


#maxProlik
REML = function(formula, y, locs, cov.fun="matern", param, nu=0.5, nugget=0, opt=NULL, geometry="plane"){

Dim = dim(locs)[2]
colnames(locs) = paste0("x", 1:Dim)
H = model.matrix(formula, data.frame(locs))
q = dim(H)[2]

# bounds for beta 
e0 = 1e-20
e1 = 1e8

# bounds for alpha
a0 = 1e-5
a1 = 6



n = length(y)

require(fields)

if(geometry=="plane"){
	d = fields::rdist(locs, locs)
}else if(geometry=="sphere"){
	d = fields::rdist.earth(locs, locs, miles=FALSE)
	if(nu>0.5){
		d = rdist.chord(locs, locs)
	}
}

if(is.null(opt)){
	if(length(param)==1){
		opt = list(method="Brent", maxit=500, lower=log(1e-10), upper=log(1e5))
	}else{
		opt = list(method="Nelder-Mead", maxit=500, lower=-Inf, upper=Inf)
	}
	
}else{

	if(!exists("method", where=opt)){
		if(length(param)==1){
			opt$method = "Brent"
		}else{
			opt$method = "Nelder-Mead"
		}
		
	}
	if(!exists("maxit", where=opt)){
		maxit=500
	}
	if(!exists("lower", where=opt)){
		if(length(param)==1){
			opt$lower = log(1e-10)
		}else{
			opt$lower = -Inf
		}
	}
	if(!exists("upper", where=opt)){
		if(length(param)==1){
			opt$upper = log(1e5)
		}else{
			opt$upper = Inf
		}		
	}

}


## transform parameters with support on [-Inf, Inf]
if(cov.fun=="matern"){
	init.val = c(log(param$phi), logit(param$nugget, 0, 1))
}else if(cov.fun=="CH"){
	init.val = c(logit(param$beta, e0, e1), logit(param$alpha, a0, a1), logit(param$nugget, 0, 1))
}else{
	stop("covariance function is not implemented")
}

if(cov.fun=="matern"){
	fit = try(optim(init.val, prologlik_matern, y=y, d=d, H=H, nu=nu,
	            method=opt$method, lower=opt$lower, upper=opt$upper,
	             control = list(fnscale = -1, maxit=opt$maxit)),
	           silent=T)
}else if(cov.fun=="CH"){
	fit = try(optim(init.val, prologlik_CH, y=y, d=d, H=H, nu=nu,
	            method=opt$method, lower=opt$lower, upper=opt$upper,
	             control = list(fnscale = -1, maxit=opt$maxit)),
	           silent=T)	
}else{
	stop("Covariance function is not implemented.")
}



if(inherits(fit, "try-error")){
	par = init.val

	if(cov.fun=="matern"){
	  phi = exp(par[1])
	  nugget = ilogit(par[2], 0, 1)
	  cormat = matern(d, phi, nu=nu)  

	}else if(cov.fun=="CH"){
	  # beta = exp(par[1])
	  #alpha = 1.0 + exp(par[2])

	  # beta = e0 + (exp(par[1])/(1+exp(par[1]))) * (e1-e0)
	  # alpha = a0 + (exp(par[2])/(1+exp(par[2]))) * (a1-a0)
	  beta = ilogit(par[1], e0, e1)
	  alpha = ilogit(par[2], a0, a1)
	  nugget = ilogit(par[3], 0, 1)
	  cormat = CH(d, beta, alpha, nu=nu)
	  
	}else {
		stop("Covariance function is not implemented")
	}

	Q = chol(cormat + diag(nugget,n))
	QInv = chol2inv(Q)
	coeff = solve(t(H)%*%QInv%*%H, t(H)%*%(QInv%*%y)) 
	ytilde = y - H%*%coeff
	sig2hat = c(t(ytilde)%*%QInv%*%ytilde/(n-q))

	success = FALSE
	cat("\noptimization error!\n")
	if(cov.fun=="matern"){
		fit = list(par=c(coeff, phi, sig2hat, nugget, success))
		names(fit$par) = c("coeff", "phi", "var", "nugget", "success")
	}else if(cov.fun=="CH"){
		fit = list(par=c(coeff, beta, alpha, sig2hat, nugget, success))
		names(fit$par) = c("coeff", "beta", "alpha", "var", "nugget", "success")
	}else if(cov.fun=="cauchy"){
		fit = list(par=c(coeff, phi, lambda, sig2hat, nugget, success))
		names(fit$par) = c("coeff", "phi", "lambda", "var", "nugget", "success")
	}
}else{
	success = TRUE
	if(cov.fun=="matern"){
	  par = fit$par 
	  phi = exp(par[1])
	  nugget = ilogit(par[2], 0, 1)
	  cormat = matern(d, phi, nu=nu)
	  Q = chol(cormat + diag(nugget,n))
	  QInv = chol2inv(Q)
	  coeff = solve(t(H)%*%QInv%*%H, t(H)%*%(QInv%*%y)) 
	  ytilde = y - H%*%coeff
	  sig2hat = t(ytilde)%*%QInv%*%ytilde/(n-q) 
	  fit$par = c(coeff, phi, c(sig2hat), nugget, success)
	  names(fit$par) = c("coeff", "phi", "var", "nugget", "success")

	}else if(cov.fun=="CH"){
		par = fit$par 
	    #beta = exp(par[1])
	    #alpha = 1.0 + exp(par[2])
	    # beta = e0 + (exp(par[1])/(1+exp(par[1]))) * (e1-e0)
	    # alpha = a0 + (exp(par[2])/(1+exp(par[2]))) * (a1-a0)
	    beta = ilogit(par[1], e0, e1)
	    alpha = ilogit(par[2], a0, a1)
	    nugget = ilogit(par[3], 0, 1)
	    cormat = CH(d, beta, alpha, nu=nu)
		Q = chol(cormat + diag(nugget,n))
	    QInv = chol2inv(Q)
	    coeff = solve(t(H)%*%QInv%*%H, t(H)%*%(QInv%*%y)) 
	    ytilde = y - H%*%coeff
	    sig2hat = t(ytilde)%*%QInv%*%ytilde/(n-q) 
	    fit$par = c(coeff, beta, alpha, c(sig2hat), nugget, success)    
		names(fit$par) = c("coeff", "beta", "alpha", "var", "nugget", "success")
	}

}



return(fit)

}










###################################################################################
############## Prediction
###################################################################################

krige = function(formula, y, locs, param, locs.pred, cov.fun="matern", nu=0.5, geometry="plane"){

n = length(y)
Dim = dim(locs)[2]
colnames(locs) = paste0("x", 1:Dim)
H = model.matrix(formula, data.frame(locs))
q = dim(H)[2]

colnames(locs.pred) = paste0("x", 1:Dim)
Hp = model.matrix(formula, data.frame(locs.pred))

if(geometry=="plane"){
	d = fields::rdist(locs, locs)
	dp = fields::rdist(locs, locs.pred)
}else if(geometry=="sphere"){
	d = fields::rdist.earth(locs, locs, miles=FALSE)
	dp = fields::rdist.earth(locs, locs.pred, miles=FALSE)
	if(nu>0.5){
		d = rdist.chord(locs, locs)
		dp = rdist.chord(locs, locs.pred)
	}
}

if(cov.fun=="matern"){

	phi = param$phi 
	sig2 = param$var 
	cormat = matern(d, phi, nu=nu)  
	cormat.pred = matern(dp, phi, nu=nu)

}else if(cov.fun=="CH"){
	beta = param$beta 
	alpha = param$alpha 
	sig2 = param$var 
	cormat = CH(d, beta, alpha, nu=nu)
	cormat.pred = CH(dp, beta, alpha, nu)
}else if(cov.fun=="cauchy"){
	phi = param$phi 
	lambda = param$lambda 
	sig2 = param$var 
	cormat = cauchy(d, phi, lambda, nu=nu)
	cormat.pred = cauchy(dp, phi, lambda, nu=nu)
}

nugget = param$nugget
coeff = param$coeff

cormat = cormat + diag(nugget, n)
# cormat.pred = cormat.pred 
Q = chol(cormat)
QInv = chol2inv(Q)

ytilde = y-H%*%coeff
Qy = backsolve(Q, ytilde, transpose=TRUE)
Qpred = backsolve(Q, cormat.pred, transpose=TRUE)

pred.mu = drop(Hp%*%coeff + crossprod(Qpred, Qy))

# pred.mu = drop(Hp%*%coeff + t(cormat.pred)%*%QInv%*%ytilde)

HRHInv = solve(t(H)%*%QInv%*%H)
Htemp = Hp - t(cormat.pred)%*%QInv%*%H
var.coeff = diag(Htemp %*% HRHInv %*% t(Htemp))
pred.var = 1 - diag(crossprod(Qpred)) + var.coeff

return(list(mean=pred.mu, var=sig2*c(pred.var)))

}



CVG <- function(x, mu, sig){
  ### Input Arguments:
  # x: a vector of observations to be predicted
  # mu: a vector of predictive means
  # sig: a vector of predictive standard errors
  ### output: nominal 95% coverage of prediction interval
  FLAG = (x>=mu-1.96*sig) & (x<=mu+1.96*sig)
  cvg = sum(FLAG)/length(FLAG)
  LCI = mean(2*1.96*sig)
  return(list(CVG=cvg, LCI=LCI))
}



