
#setwd("~/Desktop")
setwd("./code")

# data should be downloaded as described in the file readme.txt and 
# save it as a ncdf file named OCO2LtCO2v9_L3_20190601_000000_20190801_235959.nc

rm(list=ls())
library(ncdf4)

df = nc_open("OCO2LtCO2v9_L3_20190601_000000_20190801_235959.nc")

print(df)

lat = ncvar_get(df, "lat")
lon = ncvar_get(df, "lon")
time = ncvar_get(df, "time")
XCO2 = ncvar_get(df, "xco2")
nc_close(df)

## Take average over 16 day

XCO2[which(XCO2==-999999)] = NA

xco2 = apply(XCO2[1:16,,],  c(2,3), mean, na.rm=TRUE)

coord = as.matrix(expand.grid(lat, lon))
y = c(xco2)
data = data.frame(long=coord[,2], lat=coord[,1], value=y)
del = which(is.na(data$value))
dat = data[-del, ]

###### generate semivariograms
library(geoR)
variogram.plot = variog4(coords=cbind(dat$long, dat$lat), data=dat$value,
             estimator.type = "modulus", max.dist=50)
plot(variogram.plot, omnidirectional = TRUE, xlab="distance (km)")

#####

test.region = dat$long > -120 & dat$long < -100
region.lonlat = cbind(dat$long[test.region], dat$lat[test.region])

index = rep(NA, dim(region.lonlat)[1])
for(i in 1:dim(region.lonlat)[1]){
  index[i] = which(dat$long==region.lonlat[i,1] & dat$lat==region.lonlat[i,2])
}

save(index, file="./Test_region_index.RData")


load("./Test_region_index.RData")

dat.CV = dat[-index, ]

n = dim(dat.CV)[1]
m = floor(n*0.15)

set.seed(123)
indp = sample(1:n, m, replace=TRUE)
testing.rand = cbind(dat.CV$long[indp], dat.CV$lat[indp])
testing.region = cbind(dat$long[index], dat$lat[index])
site.testing = rbind(testing.region, testing.rand)

source("spmap.R")


dat.traning = dat.CV[-indp, ]
dat.testing = rbind(dat[index,], dat.CV[indp, ])

gg = spmap(dat.traning, maptype="terrain", 
           #colour=colorspace::rainbow_hcl(50),
           #box=box.limit,
           zlim=c(405,415),size=0.5, breaks=c(405,410,415), 
           zoom=2, legendTitle="XCO2\n(ppm)")
print(gg)
gg1 = gg + 
     geom_point(data=dat.testing, aes(x=long, y=lat),
                size=0.5, shape=15, color="black", alpha=1)

plot(gg1)

pdf(file="./OCO2_data_heldout.pdf", width=6.5,height=4)
print(gg1)
dev.off()

source("./Rfun_app.R")

y = dat.traning$value
site.obs = cbind(dat.traning$long, dat.traning$lat)

#####################################################################
nu=0.5
#### kriging with Matern covariance 
init.val = list(phi=168.53, nugget=0.065)
fit1 = REML(formula=~1, y=y, locs=site.obs, cov.fun="matern", 
            param=init.val, nu=nu, geometry="sphere")
param1 = as.list(fit1$par)
nugget1 = param1$var*param1$nugget

pred1 = krige(formula=~1, y=y, locs=site.obs, param=param1, 
              locs.pred=site.testing, cov.fun="matern", nu=nu, 
              geometry="sphere")

RMSPE1 = sqrt(mean((pred1$mean-dat.testing$value)^2))
measure1 = CVG(dat.testing$value, pred1$mean, (pred1$var+nugget1)^0.5)
cvg1 = measure1$CVG
ALCI1 = measure1$LCI
ind.region = seq(1,239,by=1)

RMSPE1.region = sqrt(mean((pred1$mean[ind.region]-dat.testing$value[ind.region])^2))
measure1.region = CVG(dat.testing$value[ind.region], pred1$mean[ind.region], (pred1$var[ind.region]+nugget1)^0.5)
cvg1.region = measure1.region$CVG
ALCI1.region = measure1.region$LCI

RMSPE1.MAR = sqrt(mean((pred1$mean[-ind.region]-dat.testing$value[-ind.region])^2))
measure1.MAR = CVG(dat.testing$value[-ind.region], pred1$mean[-ind.region], (pred1$var[-ind.region]+nugget1)^0.5)
cvg1.MAR = measure1.MAR$CVG
ALCI1.MAR = measure1.MAR$LCI 


#### kriging with the CH covariance 
init.val = list(beta=6216, alpha=0.3603, nugget=0.01)
fit2 = REML(formula=~1, y=y, locs=site.obs, cov.fun="CH", 
            param=init.val, nu=nu, geometry="sphere")
param2 = as.list(fit2$par)
nugget2 = param2$var*param2$nugget
pred2 = krige(formula=~1, y=y, locs=site.obs, param=param2, 
              locs.pred=site.testing, cov.fun="CH", nu=nu, 
              geometry="sphere")
RMSPE2 = sqrt(mean((pred2$mean-dat.testing$value)^2))
measure2 = CVG(dat.testing$value, pred2$mean, (pred2$var+nugget2)^0.5)
cvg2 = measure2$CVG
ALCI2 = measure2$LCI

RMSPE2.region = sqrt(mean((pred2$mean[ind.region]-dat.testing$value[ind.region])^2))
measure2.region = CVG(dat.testing$value[ind.region], pred2$mean[ind.region], (pred2$var[ind.region]+nugget2)^0.5)
cvg2.region = measure2.region$CVG
ALCI2.region = measure2.region$LCI

RMSPE2.MAR = sqrt(mean((pred2$mean[-ind.region]-dat.testing$value[-ind.region])^2))
measure2.MAR = CVG(dat.testing$value[-ind.region], pred2$mean[-ind.region], (pred2$var[-ind.region]+nugget2)^0.5)
cvg2.MAR = measure2.MAR$CVG
ALCI2.MAR = measure2.MAR$LCI 

#####################################################################
nu=1.5
#### kriging with Matern covariance 
init.val = list(phi=168.53, nugget=0.075)
fit3 = REML(formula=~1, y=y, locs=site.obs, cov.fun="matern", 
            param=init.val, nu=nu, geometry="sphere")
param3 = as.list(fit3$par)
nugget3 = param3$var*param3$nugget
pred3 = krige(formula=~1, y=y, locs=site.obs, param=param3, 
              locs.pred=site.testing, cov.fun="matern", nu=nu, 
              geometry="sphere")

RMSPE3 = sqrt(mean((pred3$mean-dat.testing$value)^2))
measure3 = CVG(dat.testing$value, pred3$mean, (pred3$var+nugget3)^0.5)
cvg3 = measure3$CVG
ALCI3 = measure3$LCI

RMSPE3.region = sqrt(mean((pred3$mean[ind.region]-dat.testing$value[ind.region])^2))
measure3.region = CVG(dat.testing$value[ind.region], pred3$mean[ind.region], (pred3$var[ind.region]+nugget3)^0.5)
cvg3.region = measure3.region$CVG
ALCI3.region = measure3.region$LCI

RMSPE3.MAR = sqrt(mean((pred3$mean[-ind.region]-dat.testing$value[-ind.region])^2))
measure3.MAR = CVG(dat.testing$value[-ind.region], pred3$mean[-ind.region], (pred3$var[-ind.region]+nugget3)^0.5)
cvg3.MAR = measure3.MAR$CVG
ALCI3.MAR = measure3.MAR$LCI 

#### kriging with New covariance 
init.val = list(beta=6216, alpha=0.3603, nugget=0.01)
fit4 = REML(formula=~1, y=y, locs=site.obs, cov.fun="CH", 
            param=init.val, nu=nu, geometry="sphere")
param4 = as.list(fit4$par)
nugget4 = param4$var*param4$nugget
pred4 = krige(formula=~1, y=y, locs=site.obs, param=param2, 
              locs.pred=site.testing, cov.fun="CH", nu=nu, 
              geometry="sphere")
RMSPE4 = sqrt(mean((pred4$mean-dat.testing$value)^2))
measure4 = CVG(dat.testing$value, pred4$mean, (pred4$var+nugget4)^0.5)
cvg4 = measure4$CVG
ALCI4 = measure4$LCI

RMSPE4.region = sqrt(mean((pred4$mean[ind.region]-dat.testing$value[ind.region])^2))
measure4.region = CVG(dat.testing$value[ind.region], pred4$mean[ind.region], (pred4$var[ind.region]+nugget4)^0.5)
cvg4.region = measure4.region$CVG
ALCI4.region = measure4.region$LCI

RMSPE4.MAR = sqrt(mean((pred4$mean[-ind.region]-dat.testing$value[-ind.region])^2))
measure4.MAR = CVG(dat.testing$value[-ind.region], pred4$mean[-ind.region], (pred4$var[-ind.region]+nugget4)^0.5)
cvg4.MAR = measure4.MAR$CVG
ALCI4.MAR = measure4.MAR$LCI 
