spmap <- function(df, 
                  maptype="terrain",
                  colours = colorspace::diverging_hsv(50),
                  zlim=NULL, breaks=NULL,
                  zoom = 10, size=1, xlim=NULL, ylim=NULL,
                  source = "google", 
                  box = NULL,
                  # color_low = "blue", color_mid="white", color_high="red",
                  mainTitle=NULL, legendTitle="ppm"){
###
# colours: 

require(sp)
require(maptools)
require(ggplot2)
require(maps)
require(scales)
require(ggmap)
  


if(is.null(box)){
  lonlat = cbind(df$long, df$lat)
  box = c(left=min(lonlat[,1]), bottom=min(lonlat[ ,2]), 
             right=max(lonlat[,1]), top=max(lonlat[,2]))
}

swfl_map = get_map(box, zoom=zoom, scale=1, maptype=maptype, source=source)

g = ggmap(swfl_map)

## plot peak surge elevation

g = g + geom_point(data=df, 
        aes(x=long, y=lat, color=value),
        size=size, shape=15, alpha=0.9) 



if(is.null(breaks)){
  g = g + scale_colour_gradientn(colours=colours,
                         limits=zlim, name=legendTitle,
                         oob=squish)
}else{
  g = g + scale_colour_gradientn(colours=colours,
                         limits=zlim, name=legendTitle,
                         oob=squish, breaks=breaks)
}



  # if(is.null(breaks)){
  #   g = g + scale_colour_gradient2(midpoint=mean(df$value), 
  #                          low=color_low, mid=color_mid,
  #                          high=color_high, 
  #                          limits=zlim, name=legendTitle,
  #                          oob=squish)
  # }else{
  #   g = g + scale_colour_gradient2(midpoint=mean(df$value), 
  #                          low=color_low, mid=color_mid,
  #                          high=color_high, 
  #                          limits=zlim, name=legendTitle,
  #                          oob=squish, breaks=breaks)
  # }

  
  # g = g + theme(plot.title = element_text(size = rel(1.2)),
  #               #panel.background = element_blank(),
  #               panel.grid.major=element_blank(),
  #               panel.grid.minor=element_blank(),
  #               legend.key.size = unit(.2, "cm"),
  #               legend.key.width = unit(1.2,"cm"),
  #               legend.box.spacing = unit(0.1, "cm"),
  #               legend.box.margin = margin(0, 0, 0, 0, "cm"),
  #               plot.margin = margin(-1, -1, 0.05, -1, "cm"),
  #               axis.text=element_text(size=14),
  #                 axis.title=element_text(size=14),
  #                 axis.title.x=element_text(size=14),
  #                 axis.text.x=element_text(size=14),
  #                 axis.title.y=element_text(size=14),
  #                 axis.text.y=element_text(size=14),
  #                 legend.text=element_text(size=14),
  #                 legend.title=element_text(size=14),
  #                 legend.position = "top"
  #               )

  g = g + theme(plot.title = element_text(size = rel(1.2)),
                #panel.background = element_blank(),
                panel.grid.major=element_blank(),
                panel.grid.minor=element_blank(),
                legend.key.size = unit(1.0, "cm"),
                legend.key.width = unit(0.2,"cm"),
                legend.box.spacing = unit(0.1, "cm"),
                legend.box.margin = margin(0, 0, 0, 0, "cm"),
                plot.margin = margin(-1, -1, 0.05, -1, "cm"),
                  axis.text=element_text(size=14),
                  axis.title=element_text(size=14),
                  axis.title.x=element_text(size=14),
                  axis.text.x=element_text(size=14),
                  axis.title.y=element_text(size=14),
                  axis.text.y=element_text(size=14),
                  legend.text=element_text(size=14),
                  legend.title=element_text(size=14)
                )
  
  g = g + labs(title=paste(mainTitle,"\n",sep=""), x="long", y="lat")
  

  
  
  return(g)
  
}
